@@ -9,9 +9,9 @@ module Tmuxinator
     class << self
       # The directory (created if needed) in which to store new projects
       def directory
-        return environment if File.directory?(environment)
-        return xdg if File.directory?(xdg)
-        return home if File.directory?(home)
+        return environment if environment?
+        return xdg if xdg?
+        return home if home?
         # No project directory specified or existant, default to XDG:
         FileUtils::mkdir_p(xdg)
         xdg
@@ -21,6 +21,10 @@ module Tmuxinator
         ENV["HOME"] + "/.tmuxinator"
       end
 
+      def home?
+        File.directory?(home)
+      end
+
       # ~/.config/tmuxinator unless $XDG_CONFIG_HOME has been configured to use
       # a custom value. (e.g. if $XDG_CONFIG_HOME is set to ~/my-config, the
       # return value will be ~/my-config/tmuxinator)
@@ -28,6 +32,10 @@ module Tmuxinator
         XDG["CONFIG"].to_s + "/tmuxinator"
       end
 
+      def xdg?
+        File.directory?(xdg)
+      end
+
       # $TMUXINATOR_CONFIG (and create directory) or "".
       def environment
         environment = ENV["TMUXINATOR_CONFIG"]
@@ -36,6 +44,10 @@ module Tmuxinator
         environment
       end
 
+      def environment?
+        File.directory?(environment)
+      end
+
       def sample
         asset_path "sample.yml"
       end
@@ -119,7 +131,7 @@ module Tmuxinator
       # Listed in search order
       # Used by `implode` and `list` commands
       def directories
-        if !environment.nil? && !environment.empty?
+        if environment?
           [environment]
         else
           [xdg, home].select { |d| File.directory? d }
@@ -181,8 +193,8 @@ module Tmuxinator
       # recursively searching 'directory'
       def project_in(directory, name)
         return nil if String(directory).empty?
-        projects = Dir.glob("#{directory}/**/*.yml").sort
-        projects.detect { |project| File.basename(project, ".yml") == name }
+        projects = Dir.glob("#{directory}/**/*.{yml,yaml}").sort
+        projects.detect { |project| File.basename(project, ".*") == name }
       end
     end
   end
